#!/usr/bin/env python
# -- coding: utf-8 --

#Copyright (C) 2007   Authors: Jacobo Tarragón Cros, Víctor Muñoz Ferrara, William Viana Soares. 
#Email addresses: tarragon[AT]aditel[DOT]org, victorm[AT]aditel[DOT]org, wviana[AT]aditel[DOT]org
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

import sys, os.path
from PyQt4 import QtGui, QtCore
from ui_mainwindow import Ui_MainWindow
from musicquizz_core import Juego
from musicquizz_config import ConfigWindow
from Jukebox import Jukebox


class MusicQuizz(QtGui.QMainWindow, Ui_MainWindow):
	#llamadme cutre si quereis pero era por tener algo :P
	MENSAJE_ABOUT = QtCore.QT_TR_NOOP("<p><h2>MusicQuizz versión 0.1</h2></p>"
			"<p><h3>Autores:</h3>"
			"<ul>"
				"<li>Jacobo Tarragón Cros: <a href='mailto:tarragon@aditel.org'>tarragon@aditel.org</a></li>"
				"<li>Víctor Muñoz Ferrara: <a href='mailto:tarragon@aditel.org'>victorm@aditel.org</a></li>"
				"<li>William Viana Soares: <a href='mailto:tarragon@aditel.org'>wviana@aditel.org</a></li>"
			"</ul></p>")
	
	MENSAJE_ABOUT = QtGui.QApplication.translate("MusicQuizz", MENSAJE_ABOUT, None, QtGui.QApplication.UnicodeUTF8)
	
	def __init__(self):

		QtGui.QMainWindow.__init__(self)

		# preparar la UI hecha con el designer
		self.setupUi(self)
		
		# motor del juego
		self.juego = Juego()
		self.ipod = Jukebox()
		
		# fine tunning
		self.botones = [self.btn_opcion0, self.btn_opcion1, self.btn_opcion2, self.btn_opcion3, 
				self.btn_opcion4, self.btn_opcion5, self.btn_opcion6, self.btn_opcion7]
		
		self.progressBar.setTextVisible(False)
		
		self.reloj_juego = QtCore.QTime(0,0,0)
		self.tiempo_juego = QtCore.QTimer()
		self.tiempo_ronda = QtCore.QTimer()
		self.tiempo_exploracion = QtCore.QTimer()
		self.retardo_ronda = QtCore.QTimer()
		self.steps_retardo = 5
		
		self.game_over = False #para saber si se han terminado todas las rondas
		self.recargar_config = False #para saber si hay que recargar la configuración
		self.pause = False # para saber si el juego está en pausa o no
		
		#Colores para mostrar si se ha acertado la canción
		#self.color_normal = self.palette()
		#self.rojo = QtGui.QPalette()
		#self.verde = QtGui.QPalette()
		#self.rojo.setColor(QtGui.QPalette.Button, QtGui.QColor(250,0,100))
		#self.verde.setColor(QtGui.QPalette.Button, QtGui.QColor(0,250,100))
		
		#self.juego.prefs.ops.opciones = 5 #numero de opciones en juego
		self.botones_juego = QtGui.QButtonGroup()
		self.botones_juego.setExclusive(False)
		
		#for i in xrange(self.juego.prefs.ops.opciones):
		for i in xrange(len(self.botones)):
			self.botones_juego.addButton(self.botones[i], i) #el segundo argumento añade un id al botón
		
		self.cargarConfiguracion()
		self.mostrarBotones()
		self.comprobarCanciones()
		
		
				
		# signals n' slots
		self.connect(self.botones_juego, QtCore.SIGNAL("buttonClicked(int)"), self.comprobarBoton)
		self.connect(self.tiempo_juego, QtCore.SIGNAL("timeout()"), self.actualizarTiempoTotal)
		self.connect(self.tiempo_ronda, QtCore.SIGNAL("timeout()"), self.actualizarTiempoRonda)
		self.connect(self.retardo_ronda, QtCore.SIGNAL("timeout()"), self.actualizarRetardo)
		self.connect(self.tiempo_exploracion, QtCore.SIGNAL("timeout()"), self.actualizarTiempoExploracion)
		self.connect(self.btnPlay, QtCore.SIGNAL("clicked(bool)"), self.playPause)
		self.connect(self.actionPreferencias, QtCore.SIGNAL("activated()"), self.mostrarConfiguracion)
		self.connect(self.actionTerminar_Juego, QtCore.SIGNAL("activated()"), self.terminarJuego)
		self.connect(self.actionNuevo_Juego, QtCore.SIGNAL("activated()"), self.nuevoJuego)
		self.connect(self.actionSalir, QtCore.SIGNAL("activated()"), self, QtCore.SLOT("close()"))
		self.connect(self.actionExplorar_Coleccion, QtCore.SIGNAL("activated()"), self.explorarColeccion)
		self.connect(self.actionAcerca_de, QtCore.SIGNAL("activated()"), self.acerca_de)
	
	
	# SLOTS *************************

	def actualizarTiempoTotal(self):
		self.reloj_juego = self.reloj_juego.addSecs(1)
		self.lblTiempo.setText(self.reloj_juego.toString("hh:mm:ss"))
	
	
	def actualizarTiempoRonda(self):
		t = self.progressBar.value()
		if t == 0:
			if not self.juego.finRonda(None, 0):
				self.terminarJuego()
			else:
				self.ocultarBotones()
				self.ipod.stop()
				self.tiempo_ronda.stop()
				self.mostrarVerdadera()
				self.actualizarLabels()
				self.retardo_ronda.start(100)	
		else:
			self.progressBar.setValue(t-1)
			
			
	def actualizarRetardo(self):
		""" Añade un retardo entre rondas. """
		if self.steps_retardo > 0:
			self.steps_retardo -= 1
		else:
			self.retardo_ronda.stop()
			self.steps_retardo = 5
			
			self.botones[self.juego.verdadera].setCheckable(True)
			#recuperar el color normal de los botones 
			#for i in xrange(self.juego.prefs.ops.opciones):
				#self.botones[i].setPalette(self.color_normal)
			
			if self.game_over:
				self.terminarJuego()
				self.game_over = False
			else:
				self.nuevaRonda()
	
	def actualizarTiempoExploracion(self):
		self.progress_dialog.setValue(101)
	
	
	def playPause(self):
		if self.tiempo_ronda.isActive(): # pausamos
			
			#ocultar todos los botones menos el play
			self.ocultarBotones() 
			self.btnPlay.setEnabled(True)  
			
			self.ipod.stop()
			self.tiempo_ronda.stop()
			self.game_over = not self.juego.finRonda(None, 0)
			self.btnPlay.setText("&Play")
			self.pause = True
			if self.game_over:
				self.terminarJuego()
		elif self.pause == True: # ponemos a play
			self.actualizarLabels()
			self.tiempo_juego.start(1000)
			self.nuevaRonda()			
			self.btnPlay.setText("&Pause")
			self.pause = False
		else: #empezamos una nueva partida
			self.comprobarCanciones()
			if self.recargar_config:
				self.cargarConfiguracion()
				self.recargar_config = False
			self.actualizarLabels()
			self.tiempo_juego.start(1000)
			self.nuevaRonda()			
			self.btnPlay.setText("&Pause")
			
			
			
	
	def comprobarBoton(self):
		if self.tiempo_ronda.isActive():
			self.ocultarBotones()
			self.ipod.stop()
			self.tiempo_ronda.stop()
			
			#FIXME: finRonda debería devolver True si no quedan más rondas, así evitaría el not de abajo que queda un poco raro
			self.game_over = not self.juego.finRonda(self.botones_juego.checkedId(), self.progressBar.value())
			
			#if self.juego.comprobarAcierto(self.botones_juego.checkedId()):
				#self.botones[self.botones_juego.checkedId()].setPalette(self.verde)
                        #else:
				#self.botones[self.botones_juego.checkedId()].setPalette(self.rojo)
				
			self.botones[self.botones_juego.checkedId()].setChecked(False)
			self.mostrarVerdadera()
			self.actualizarLabels()
			self.retardo_ronda.start(100)	
	
	def mostrarConfiguracion(self):
		if self.tiempo_ronda.isActive():
			self.playPause()#Pausamos
		self.juego.prefs.recargarPreferencias()
		config_window = ConfigWindow(self.juego.prefs)
		guardar = config_window.exec_()
		if guardar:
			#self.juego.prefs.guardar()
			self.recargar_config = True
			return True
		return False
	
	def explorarColeccion(self):
		#Esto necesita un buen cambio y va a dar algún que otro dolor de cabeza
		if self.tiempo_ronda.isActive():
			self.terminarJuego() 
		texto = QtGui.QApplication.translate("MusicQuizz", "Debe especificar al menos un directorio válido con canciones", None, QtGui.QApplication.UnicodeUTF8)
		
		if len(self.juego.prefs.ops.directorios) == 0:
			while len(self.juego.prefs.ops.directorios) == 0:
				resp = QtGui.QMessageBox.warning(self,self.tr("Aviso - MusicQuizz"), texto, self.tr("Aceptar"), self.tr("Cancelar"))
				if resp == 0:
					if self.mostrarConfiguracion():
						print "[MusicQuizz] Explorando colección..."
						self.juego.coleccion.explorarColeccion(self.juego.prefs.ops.directorios, self.juego.prefs.ops.fichero)
						print "[MusicQuizz] Exploración completa"
				else:
					break
		else:
			print "[MusicQuizz] Explorando colección..."
			self.juego.coleccion.explorarColeccion(self.juego.prefs.ops.directorios, self.juego.prefs.ops.fichero)
			print "[MusicQuizz] Exploración completa"
	
	# Uso interno ***********************
	
	def comprobarCanciones(self):
		texto = QtGui.QApplication.translate("MusicQuizz", "El fichero de canciones está vacio o no existe. Pulse Aceptar para explorar la colección", None, QtGui.QApplication.UnicodeUTF8)

		if not os.path.exists(self.juego.prefs.ops.fichero):
			resp = QtGui.QMessageBox.warning(self,self.tr("Aviso - MusicQuizz"), texto, self.tr("Aceptar"), self.tr("Cancelar"))
			if resp == 0:
				print "[MusicQuizz] Explorando colección..."		
				self.explorarColeccion()
				print "[MusicQuizz] Exploración completa"
			else:
				sys.exit(-1)
		else:
			f = open(self.juego.prefs.ops.fichero)
			if len(f.read()) == 0:
				resp = QtGui.QMessageBox.warning(self,self.tr("Aviso - MusicQuizz"), texto, self.tr("Aceptar"), self.tr("Salir"))
				if resp == 0:
					print "[MusicQuizz] Explorando colección..."
					self.explorarColeccion()
					print "[MusicQuizz] Exploración completa"
				else:
					sys.exit(0)
		
	
	def nuevaRonda(self):
		self.ocultarBotones()
		self.progressBar.setValue((self.juego.prefs.ops.tiempo-1)*10)
		canciones = self.juego.prepararRonda()
		self.cambiarTextoBotones(canciones)
		self.mostrarBotones()
		self.ipod.play(self.juego.coleccion[self.juego.idx_verdadera].ruta)
		self.tiempo_ronda.start(100)
		
	
	def nuevoJuego(self):
		if self.tiempo_ronda.isActive():
			self.terminarJuego()
			self.playPause()
		else:
			self.playPause()
			
	def terminarJuego(self):

		self.ipod.stop()
		self.tiempo_ronda.stop()
		self.tiempo_juego.stop()
		
		self.reloj_juego = QtCore.QTime(0,0,0)
		self.lblTiempo.setText(self.reloj_juego.toString("hh:mm:ss"))
		
		self.juego.reset()
		self.ocultarBotones()
		self.btnPlay.setText("&Play")
		self.btnPlay.setEnabled(True)

	def cargarConfiguracion(self):
		
		self.juego.prefs.recargarPreferencias()
		
		self.progressBar.setRange(0,(self.juego.prefs.ops.tiempo-1)*10)
		self.progressBar.setValue((self.juego.prefs.ops.tiempo-1)*10)
		self.segundos_ronda = self.juego.prefs.ops.tiempo
		
		for i in xrange(self.juego.prefs.ops.opciones):
			self.botones[i].show()
		
		for i in xrange(self.juego.prefs.ops.opciones, len(self.botones)):
			self.botones[i].hide()
			
		self.resize(QtCore.QSize(QtCore.QRect(0,0,358,320 + (self.juego.prefs.ops.opciones - 2)* 25).size()).expandedTo(self.minimumSizeHint()))
	
		
	# Funciones para manipular atributos del widget ******************
	
	def mostrarBotones(self):
		self.btnPlay.setEnabled(True)
		for i in xrange(self.juego.prefs.ops.opciones):
			self.botones[i].setEnabled(True)
	
	def actualizarLabels(self):
		self.lblPuntuacion.setText( self.juego.strPuntos() )
		self.lblAciertos.setText( self.juego.promedioAciertos(self.game_over) )
		self.lblRonda.setText( self.juego.strRonda() )
		
	def ocultarBotones(self):
		self.btnPlay.setEnabled(False)
		for i in xrange(self.juego.prefs.ops.opciones):
			self.botones[i].setEnabled(False)
	
	def mostrarVerdadera(self):
		self.botones[self.juego.verdadera].setCheckable(False)
		self.botones[self.juego.verdadera].setEnabled(True)
	
	def cambiarTextoBotones(self, canciones):
		for i in xrange(len(canciones)):
			label = ""
			if self.juego.prefs.ops.titulo: 
				label += canciones[i].titulo
			if self.juego.prefs.ops.autor:
				if label: label += " - "
				label +=  canciones[i].autor
			#else:
				#label = canciones[i].titulo + " - " + canciones[i].autor
				
			self.botones[i].setText(label)
		
	def acerca_de(self):
		QtGui.QMessageBox.about(self, "Acerca de MusicQuizz", MusicQuizz.MENSAJE_ABOUT)

def main(argv):
	"""Esta función es para llamarla desde el script q se instala en /usr/bin
	así los imports de mq.py pueden seguir siendo relativos."""
	app = QtGui.QApplication(argv)
	ui = MusicQuizz()
	ui.show()
	sys.exit(app.exec_())


if __name__ == "__main__":
	main(sys.argv)
